%%% DESENSITIZATION_ANALYSIS.M
%%% This program outputs quantitative measures of neutrophil migration
%%% using an Excel or other spreadsheet file containing the XY-positions of
%%% the various tracked neutrophils and a directory of the fluorescence image 
%%% sequences of the Microjets Device during the experiment.  The data 
%%% should be arranged in columns according to the format:
%%%   [(Image Name)(Image Plane)(Track #)(X)(Y)] 
%%% for the program to run properly.  Other formats are possible
%%% but require modification of the code.  This program also requires the
%%% use of the function MAKE_EXP_LINESCAN_ARRAY_FXN.M.  This essential
%%% M-file should be placed in the same directory as
%%% DESENSITIZATION_ANALYSIS.M.  
% Copyright 2009 - Thomas M. Keenan, University of Wisconsin

clear

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load Cell Position Data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load_cell_data_ques=questdlg('Would you like to load cell data from an Excel file or existing *.mat file?','Load Cell Data','Excel','*.mat','*.mat') ;
if load_cell_data_ques(1)=='E'
    [CellFileName,CellPathName] = uigetfile('*.xls','Choose the Cell Position Excel File')
    CellFileName_load=strcat(CellPathName,'/',CellFileName);
    data = xlsread(CellFileName_load); % Make sure data has columns [(Image Name)(Image Plane)(Track #)(X)(Y)(Distance)(Velocity)(Absolute Angle)(Distance To Origin)(Delta X)(Delta Y)]
    
    % Sort data into cell array with one cell array for each cell
    data_size=size(data);
    num_cells=data(data_size(1),2); % gets the value of the last track number (i.e. number of cells)
    for i=1:num_cells
        a=find(data(:,2)==i); % searches track number column for all cell positions associated with a particular track number
        size_a=size(a);
        data_start(i,1)=a(1);
        data_end(i,2)=a(size_a(1));
        sorted_data{i,1}=data(data_start(i,1):data_end(i,2),:);
        clear a size_a
    end
    
    % Save cell position data
    save_ques=questdlg('Would you like to save the sorted cell position data?','Save','Yes','No','Yes');
    if save_ques(1)=='Y'
        save_path=uigetdir('','Select the SAVE destination:')
        prompt = {'Enter filename:'};
        dlg_title = 'Save'; 
        num_lines= 1; 
        [pathstr,name,ext,versn] = fileparts(save_path); 
        def     = {name};
        save_name = inputdlg(prompt,dlg_title,num_lines,def); %defines name for file
        cd(save_path)                           %Changes directory to user-chosen dir
        save (save_name{1,1}, 'sorted_data');   %Saves sorted_data under "filename"
    end

else %Load previously saved cell position data *.mat file
    [CellFileName,CellPathName] = uigetfile('*.mat','Choose the Cell Position *.MAT File')
    load(strcat(CellPathName,'/',CellFileName));
end

sorted_data_size=size(sorted_data);
num_cells=sorted_data_size(1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make or load exp linescan array from fluorescence images %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
new_exp_array_ques=questdlg('Would you like to make a new exp_linescan_array or load an existing one?','Exp_Linescan_Array','New','Existing','New');
if new_exp_array_ques(1)=='N'
    [exp_linescan_array,exp_pressure,max_I,min_I,rec_crop_last_def]=make_exp_linescan_array_fxn;
else
    [FileName,PathName] = uigetfile('*.mat','Choose the Exp_Linescan_Array file')
    load(strcat(PathName,'/',FileName));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prompt user for information %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
num_pixels_input=inputdlg('How many pixels should be used to calculate the gradient across the cell?');
num_pixels=(str2num(num_pixels_input{1,1}));
half_range = floor(num_pixels/2);

transition_pts_input=inputdlg('In what frames do major stages end (use spaces to separate the frame numbers for each major stage and remember to include the last time point)?');
transition_pts=str2num(transition_pts_input{1,1});
num_transition_pts=size(transition_pts);

% Time Vector
time_interval_input=inputdlg('How much time (in minutes) separates each time point?');
time_interval=(str2num(time_interval_input{1,1})); % minutes

% Pixels to um
pixels_to_um_input=inputdlg('How many pixels/um?');
pixels_to_um=(str2num(pixels_to_um_input{1,1}));

% Choose which cells to include
chosen_cells_input=inputdlg('Which cells should be included in the output figures/analyses? Use colons (:) to select ranges of values');
chosen_cells=(str2num(chosen_cells_input{1,1}));
num_chosen_cells=size(chosen_cells);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Crop all data to just gradient region %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Define Gradient Region from image
%     [disp_img_name,disp_img_path]=uigetfile({'*.tif';'*.jpg';'*.bmp';'*.gif'},'Select the image to be used to define the gradient region')
%     disp_img=imread(strcat(disp_img_path,'\',disp_img_name));
%     disp_img_size=size(disp_img);
%     [cropped_disp_img,cropped_rect]=imcrop(disp_img,[]);
%     x_min=floor(cropped_rect(1));
%     x_span=ceil(cropped_rect(3));
%     close all

%% Define Gradient Region from cell positions
for i=1:num_cells
    cell_position_max(i,1)=max(sorted_data{i,1}(:,3));
    cell_position_min(i,1)=min(sorted_data{i,1}(:,3));
end
x_min=min(cell_position_min)-half_range;
x_span=max(cell_position_max)-x_min+half_range;

% Crop Exp_linescan_array (assumes initial exp_linescan_array included entire x-axis)
cropped_exp_linescan_array=exp_linescan_array(x_min:(x_min+x_span),:);

% Make x origin of cell position data left side of gradient region
cropped_sorted_data=sorted_data;
max_timepoints=1;
for i=1:num_cells
    size_data_set=size(sorted_data{i,1});
    num_timepoints=size_data_set(1);
    subtraction_matrix=ones(num_timepoints,1)*x_min;
    cropped_sorted_data{i,1}(:,3)=sorted_data{i,1}(:,3)-subtraction_matrix;
    if num_timepoints>max_timepoints;
        max_timepoints=num_timepoints;
    end
end
time_vector=(time_interval:time_interval:time_interval*max_timepoints)'; % minutes

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate Quantitative Measures of Cell Movement %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i=chosen_cells %counts through chosen cells
    
    cropped_sorted_data_size(i,:)=size(cropped_sorted_data{i,1},1); % number of timepoints in data set for each cell
    
    
    for j=1:cropped_sorted_data_size(i,1) % counts through each time point
	% Store XY positions of each cell at each time point
	Results_XY_positions(cropped_sorted_data{i,1}(j,1),i,:)=cropped_sorted_data{i,1}(j,3:4)./pixels_to_um; % um 
	
	% Calculate concentration (Relative Intensity) at each cell location
	Results_conc_each_cell(cropped_sorted_data{i,1}(j,1),i)=cropped_exp_linescan_array(cropped_sorted_data{i,1}(j,3),cropped_sorted_data{i,1}(j,1));

	% Calculate the gradient across each cell by making a num_pixels wide box for linear regression analysis
		% Accomodates for cell being too close to one wall of the gradient chamber             
		if (cropped_sorted_data{i,1}(j,3)+half_range  > x_span)
                 	x_poly_vector=(cropped_sorted_data{i,1}(j,3)-half_range:cropped_sorted_data{i,1}(j,3))';
                	y_poly_vector=cropped_exp_linescan_array((cropped_sorted_data{i,1}(j,3)-half_range:cropped_sorted_data{i,1}(j,3)),cropped_sorted_data{i,1}(j,1));
             	
		% Accomodates for cell being too close to the other wall of the gradient chamber
		elseif (cropped_sorted_data{i,1}(j,3)-(num_pixels/2) <=0)
                	x_poly_vector=(cropped_sorted_data{i,1}(j,3):cropped_sorted_data{i,1}(j,3)+half_range)';
                 	y_poly_vector=cropped_exp_linescan_array((cropped_sorted_data{i,1}(j,3):cropped_sorted_data{i,1}(j,3)+half_range),cropped_sorted_data{i,1}(j,1));

             	% Pulls X and Conc data to calculate gradient for all cells not within num_pixels range of wall
		else                 	
                 	x_poly_vector=(cropped_sorted_data{i,1}(j,3)-half_range:cropped_sorted_data{i,1}(j,3)+half_range)';
                	y_poly_vector=cropped_exp_linescan_array((cropped_sorted_data{i,1}(j,3)-half_range:cropped_sorted_data{i,1}(j,3)+half_range),cropped_sorted_data{i,1}(j,1));
		end

             m=polyfit(x_poly_vector,y_poly_vector,1);

	% Store slope of gradient around cell ([=] Relative Intensity/um)
        Results_grad_each_cell(cropped_sorted_data{i,1}(j,1),i)=m(1)*pixels_to_um;
     end
end

% Calculate average total velocity for each stage
Results_XY_positions_size=size(Results_XY_positions);
for i = 2:Results_XY_positions_size(1)
	Results_XY_positions_delta(i,:,:)=Results_XY_positions(i,:,:)-Results_XY_positions(i-1,:,:);
end
Results_XY_positions_delta(1,:,:)=zeros(1,Results_XY_positions_size(2),Results_XY_positions_size(3));

Results_Total_Velocities=realsqrt(Results_XY_positions_delta(:,chosen_cells,1).^2 + Results_XY_positions_delta(:,chosen_cells,2).^2)/time_interval; % um/min
Results_X_Velocities=Results_XY_positions_delta(:,chosen_cells,1)/time_interval;	% um/min



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot Calculated Quantitative Measures %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set figure sizes
SS=get(0,'ScreenSize');
width=SS(3)*(0.97);
height=SS(4)*(0.85);
left=SS(3)*(0.03)/2;
bottom=SS(4)*(0.15)/3;
fig_size=[left bottom width height];



%%% Plot cell migration trace overlays
   
for i=1:num_transition_pts(2) % separate trace overlays for each stage
    if i==1
        xstart=1;
    else
        xstart=transition_pts(i-1)+1;
    end
    xend=transition_pts(i);
    
    for m=xstart:xend
        Overlay_Xs(m,:)=(Results_XY_positions(m,chosen_cells,1)-Results_XY_positions(xstart,chosen_cells,1));
        Overlay_Ys(m,:)=(Results_XY_positions(m,chosen_cells,2)-Results_XY_positions(xstart,chosen_cells,2));	
    end
    
    figure('Position',fig_size);
    line_width=4;
    plot(Overlay_Xs(xstart:xend,:),Overlay_Ys(xstart:xend,:),'r-','LineWidth',line_width)
    hold on
    title(strcat('Stage ', num2str(i)),'FontWeight','bold');
    X_limit_1=max(max(Overlay_Xs));
    X_limit_2=min(min(Overlay_Xs));
    X_limit=max(abs([X_limit_1 X_limit_2]));
    Y_limit_1=max(max(Overlay_Ys));
    Y_limit_2=min(min(Overlay_Ys));
    Y_limit=max(abs([Y_limit_1 Y_limit_2]));
    Max_limit=max([X_limit Y_limit]);
    xlim([-1.1*X_limit 1.1*X_limit])
    ylim([-1.1*Y_limit 1.1*Y_limit])
    plot([-1.1*Max_limit; 1.1*Max_limit],[0;0],'k:','LineWidth',3);
    plot([0;0],[-1.1*Max_limit; 1.1*Max_limit],'k:','LineWidth',3);
    xlabel('\bfX (\mum)')
    ylabel('\bfY (\mum)')
    set(gca,'LineWidth',3)
    axis equal
    hold off

plot_transitions(:,i)=[-1000;1000];
% Calculate stage velocity averages
Results_Total_Velocity_Averages_per_Stage(i,:)=mean(Results_Total_Velocities(xstart:xend,:),1);
Results_X_Velocity_Averages_per_Stage(i,:)=mean(Results_X_Velocities(xstart:xend,:),1);
end
        
        

%%% Plot conc and gradient cell sees as it migrates 
X_positions=Results_XY_positions(:,:,1);

for i=1:num_chosen_cells(2)
    %%% Plot grad each cell sees and position vs. time 
    figure('Position',fig_size);
    y_max=max(max(Results_grad_each_cell(:,chosen_cells(i))));
    y_min=min(min(Results_grad_each_cell(:,chosen_cells(i))));
    y_interval=(y_max-y_min)/10;
    [AX,H1,H2]=plotyy(time_vector,Results_grad_each_cell(:,chosen_cells(i)),time_vector,X_positions(:,chosen_cells(i)));
    hold on
    plot([transition_pts*time_interval;transition_pts*time_interval], plot_transitions,'r','LineWidth',1);
    set(get(AX(1),'Ylabel'),'String','\bf\DeltaG(\mum^-^1)') 
    set(get(AX(2),'Ylabel'),'String','X-position(\mum)') 
    set(H1,'LineStyle','-')
    set(H2,'LineStyle',':')
    legend(strcat('Cell ',num2str(chosen_cells(i))));
    %set(AX(1),'YTick',[y_min: y_interval : 1.1*y_max])
    %set(AX(2),'YTick',[0: (max(max(X_positions))/10) : 1.1*max(max(X_positions))])
    %set(AX(2),'YLim',[0 1.1*max(max(X_positions))]);
    %ylim([y_min y_max]);
    hold off
	    
    %%% Plot conc each cell sees and position vs. time
    figure('Position',fig_size);
    [AXa,Ha1,Ha2]=plotyy(time_vector,Results_conc_each_cell(:,chosen_cells(i)),time_vector,X_positions(:,chosen_cells(i)));            
    hold on
    plot([transition_pts*time_interval;transition_pts*time_interval], plot_transitions,'r','LineWidth',1);
    set(get(AXa(1),'Ylabel'),'String','\bfC/Co') 
    set(get(AXa(2),'Ylabel'),'String','X-position(\mum)') 
    set(AXa(1),'YTick',[0:.1:1])
    %set(AXa(2),'YTick',[0: (max(max(X_positions))/10) : 1.1*max(max(X_positions))])
    %set(AXa(2),'YLim',[0 1.1*max(max(X_positions))]);
    set(Ha1,'LineStyle','-')
    set(Ha2,'LineStyle',':')
    ylim([0 1]);
    legend(strcat('Cell ',num2str(chosen_cells(i))));  	
    hold off
end



%% Bar graph of Average Total Velocity and Average X-Velocity for each stage
Results_Total_Velocity_Stage_Average=mean(Results_Total_Velocity_Averages_per_Stage,2);
Results_X_Velocity_Stage_Average=mean(Results_X_Velocity_Averages_per_Stage,2);

% Plot average Euclidean velocity for the selected cells during each stage
figure
bar(Results_Total_Velocity_Stage_Average)
hold on
title('\bfAverage Stage Euclidean Velocity for Selected Cells') 
xlabel('\bfStage #')
ylabel('\bfAverage Euclidean Velocity (\mum/min)')
hold off

% Plot average X-velocity for the selected cells during each stage
figure
bar(Results_X_Velocity_Stage_Average)
hold on
title('\bfAverage Stage X-velocity for Selected Cells') 
xlabel('\bfStage #')
ylabel('\bfAverage X-Velocity (\mum/min)')
hold off

% Output velocity Data to the command window
Average_Euclidean_Velocities=Results_Total_Velocity_Averages_per_Stage
Average_X_Velocities=Results_X_Velocity_Averages_per_Stage
